@@ -270,7 +270,7 @@ gdm_session_direct_handle_setup_failed (GdmSessionDirect *session,
 
         g_debug ("GdmSessionDirect: Emitting 'setup-failed' signal");
 
-        _gdm_session_setup_failed (GDM_SESSION (session), NULL);
+        _gdm_session_setup_failed (GDM_SESSION (session), text);
 
         return DBUS_HANDLER_RESULT_HANDLED;
 }
@@ -316,7 +316,7 @@ gdm_session_direct_handle_reset_failed (GdmSessionDirect *session,
 
         g_debug ("GdmSessionDirect: Emitting 'reset-failed' signal");
 
-        _gdm_session_reset_failed (GDM_SESSION (session), NULL);
+        _gdm_session_reset_failed (GDM_SESSION (session), text);
 
         return DBUS_HANDLER_RESULT_HANDLED;
 }
@@ -334,6 +334,7 @@ gdm_session_direct_handle_authenticated (GdmSessionDirect *session,
         dbus_connection_send (connection, reply, NULL);
         dbus_message_unref (reply);
 
+        session->priv->is_authenticated = TRUE;
         _gdm_session_authenticated (GDM_SESSION (session));
 
         return DBUS_HANDLER_RESULT_HANDLED;
@@ -361,7 +362,8 @@ gdm_session_direct_handle_authentication_failed (GdmSessionDirect *session,
 
         g_debug ("GdmSessionDirect: Emitting 'authentication-failed' signal");
 
-        _gdm_session_authentication_failed (GDM_SESSION (session), NULL);
+        session->priv->is_authenticated = FALSE;
+        _gdm_session_authentication_failed (GDM_SESSION (session), text);
 
         return DBUS_HANDLER_RESULT_HANDLED;
 }
@@ -406,7 +408,7 @@ gdm_session_direct_handle_authorization_failed (GdmSessionDirect *session,
 
         g_debug ("GdmSessionDirect: Emitting 'authorization-failed' signal");
 
-        _gdm_session_authorization_failed (GDM_SESSION (session), NULL);
+        _gdm_session_authorization_failed (GDM_SESSION (session), text);
 
         return DBUS_HANDLER_RESULT_HANDLED;
 }
@@ -451,7 +453,7 @@ gdm_session_direct_handle_accreditation_failed (GdmSessionDirect *session,
 
         g_debug ("GdmSessionDirect: Emitting 'accreditation-failed' signal");
 
-        _gdm_session_accreditation_failed (GDM_SESSION (session), NULL);
+        _gdm_session_accreditation_failed (GDM_SESSION (session), text);
 
         return DBUS_HANDLER_RESULT_HANDLED;
 }
@@ -1158,18 +1160,6 @@ session_worker_message (DBusConnection *connection,
                 return gdm_session_direct_handle_accredited (session, connection, message);
         } else if (dbus_message_is_method_call (message, GDM_SESSION_DBUS_INTERFACE, "AccreditationFailed")) {
                 return gdm_session_direct_handle_accreditation_failed (session, connection, message);
-        } else if (dbus_message_is_method_call (message, GDM_SESSION_DBUS_INTERFACE, "Authenticated")) {
-                return gdm_session_direct_handle_authenticated (session, connection, message);
-        } else if (dbus_message_is_method_call (message, GDM_SESSION_DBUS_INTERFACE, "AuthenticationFailed")) {
-                return gdm_session_direct_handle_authentication_failed (session, connection, message);
-        } else if (dbus_message_is_method_call (message, GDM_SESSION_DBUS_INTERFACE, "Authorized")) {
-                return gdm_session_direct_handle_authorized (session, connection, message);
-        } else if (dbus_message_is_method_call (message, GDM_SESSION_DBUS_INTERFACE, "AuthorizationFailed")) {
-                return gdm_session_direct_handle_authorization_failed (session, connection, message);
-        } else if (dbus_message_is_method_call (message, GDM_SESSION_DBUS_INTERFACE, "Accredited")) {
-                return gdm_session_direct_handle_accredited (session, connection, message);
-        } else if (dbus_message_is_method_call (message, GDM_SESSION_DBUS_INTERFACE, "AccreditationFailed")) {
-                return gdm_session_direct_handle_accreditation_failed (session, connection, message);
         } else if (dbus_message_is_method_call (message, GDM_SESSION_DBUS_INTERFACE, "UsernameChanged")) {
                 return gdm_session_direct_handle_username_changed (session, connection, message);
         } else if (dbus_message_is_method_call (message, GDM_SESSION_DBUS_INTERFACE, "SessionStarted")) {
@@ -1569,11 +1559,7 @@ worker_exited (GdmSessionWorkerJob *job,
         g_debug ("GdmSessionDirect: Worker job exited: %d", code);
 
         if (!session->priv->is_authenticated) {
-                char *msg;
-
-                msg = g_strdup_printf (_("worker exited with status %d"), code);
-                _gdm_session_authentication_failed (GDM_SESSION (session), msg);
-                g_free (msg);
+                _gdm_session_authentication_failed (GDM_SESSION (session), NULL);
         } else if (session->priv->is_running) {
                 _gdm_session_session_exited (GDM_SESSION (session), code);
         }
@@ -1587,11 +1573,7 @@ worker_died (GdmSessionWorkerJob *job,
         g_debug ("GdmSessionDirect: Worker job died: %d", signum);
 
         if (!session->priv->is_authenticated) {
-                char *msg;
-
-                msg = g_strdup_printf (_("worker exited with status %d"), signum);
-                _gdm_session_authentication_failed (GDM_SESSION (session), msg);
-                g_free (msg);
+                _gdm_session_authentication_failed (GDM_SESSION (session), NULL);
         } else if (session->priv->is_running) {
                 _gdm_session_session_died (GDM_SESSION (session), signum);
         }
@@ -1629,6 +1611,19 @@ start_worker (GdmSessionDirect *session)
 static void
 stop_worker (GdmSessionDirect *session)
 {
+        g_signal_handlers_disconnect_by_func (session->priv->job,
+                                              G_CALLBACK (worker_stopped),
+                                              session);
+        g_signal_handlers_disconnect_by_func (session->priv->job,
+                                              G_CALLBACK (worker_started),
+                                              session);
+        g_signal_handlers_disconnect_by_func (session->priv->job,
+                                              G_CALLBACK (worker_exited),
+                                              session);
+        g_signal_handlers_disconnect_by_func (session->priv->job,
+                                              G_CALLBACK (worker_died),
+                                              session);
+
         cancel_pending_query (session);
 
         if (session->priv->worker_connection != NULL) {
@@ -1637,6 +1632,7 @@ stop_worker (GdmSessionDirect *session)
         }
 
         gdm_session_worker_job_stop (session->priv->job);
+        g_object_unref (session->priv->job);
         session->priv->job = NULL;
 }
 
@@ -1957,9 +1953,12 @@ setup_session_environment (GdmSessionDirect *session)
                                                      "GDM_LANG",
                                                      get_language_name (session));
 
-        gdm_session_direct_set_environment_variable (session,
-                                                     "GDM_KEYBOARD_LAYOUT",
-                                                     get_layout_name (session));
+        if (strcmp (get_layout_name (session),
+                    get_default_layout_name (session)) == 0) {
+                gdm_session_direct_set_environment_variable (session,
+                                                             "GDM_KEYBOARD_LAYOUT",
+                                                             get_layout_name (session));
+        }
 
         gdm_session_direct_set_environment_variable (session,
                                                      "DISPLAY",
