@@ -67,6 +67,8 @@ struct GdmUserChooserWidgetPrivate
         guint           show_user_auto : 1;
         guint           show_normal_users : 1;
 
+        guint           has_user_other : 1;
+
         guint           load_idle_id;
 };
 
@@ -83,6 +85,9 @@ static void     gdm_user_chooser_widget_finalize    (GObject                   *
 
 G_DEFINE_TYPE (GdmUserChooserWidget, gdm_user_chooser_widget, GDM_TYPE_CHOOSER_WIDGET)
 
+static void     add_user_other    (GdmUserChooserWidget *widget);
+static void     remove_user_other (GdmUserChooserWidget *widget);
+
 static int
 get_font_height_for_widget (GtkWidget *widget)
 {
@@ -122,13 +127,41 @@ get_icon_height_for_widget (GtkWidget *widget)
         return height;
 }
 
+static void
+update_other_user_visibility (GdmUserChooserWidget *widget)
+{
+        if (!widget->priv->show_user_other) {
+                if (widget->priv->has_user_other) {
+                        remove_user_other (widget);
+                }
+
+                return;
+        }
+
+        if (gdm_chooser_widget_get_number_of_items (GDM_CHOOSER_WIDGET (widget)) == 1) {
+                /* we hide the Other user if it's the last one, and we show it
+                 * if there's another user */
+                if (widget->priv->has_user_other) {
+                        remove_user_other (widget);
+                } else {
+                        add_user_other (widget);
+                }
+        }
+}
+
 static void
 add_user_other (GdmUserChooserWidget *widget)
 {
+        widget->priv->has_user_other = TRUE;
         gdm_chooser_widget_add_item (GDM_CHOOSER_WIDGET (widget),
                                      GDM_USER_CHOOSER_USER_OTHER,
                                      NULL,
-                                     _("Other..."),
+                                     /* translators: This option prompts
+                                      * the user to type in a username
+                                      * manually instead of choosing from
+                                      * a list.
+                                      */
+                                     C_("user", "Other..."),
                                      _("Choose a different account"),
                                      0,
                                      FALSE,
@@ -146,6 +179,7 @@ add_user_guest (GdmUserChooserWidget *widget)
                                      0,
                                      FALSE,
                                      TRUE);
+        update_other_user_visibility (widget);
 }
 
 static void
@@ -159,11 +193,13 @@ add_user_auto (GdmUserChooserWidget *widget)
                                      0,
                                      FALSE,
                                      TRUE);
+        update_other_user_visibility (widget);
 }
 
 static void
 remove_user_other (GdmUserChooserWidget *widget)
 {
+        widget->priv->has_user_other = FALSE;
         gdm_chooser_widget_remove_item (GDM_CHOOSER_WIDGET (widget),
                                         GDM_USER_CHOOSER_USER_OTHER);
 }
@@ -173,6 +209,7 @@ remove_user_guest (GdmUserChooserWidget *widget)
 {
         gdm_chooser_widget_remove_item (GDM_CHOOSER_WIDGET (widget),
                                         GDM_USER_CHOOSER_USER_GUEST);
+        update_other_user_visibility (widget);
 }
 
 static void
@@ -180,6 +217,7 @@ remove_user_auto (GdmUserChooserWidget *widget)
 {
         gdm_chooser_widget_remove_item (GDM_CHOOSER_WIDGET (widget),
                                         GDM_USER_CHOOSER_USER_AUTO);
+        update_other_user_visibility (widget);
 }
 
 void
@@ -190,11 +228,7 @@ gdm_user_chooser_widget_set_show_user_other (GdmUserChooserWidget *widget,
 
         if (widget->priv->show_user_other != show_user) {
                 widget->priv->show_user_other = show_user;
-                if (show_user) {
-                        add_user_other (widget);
-                } else {
-                        remove_user_other (widget);
-                }
+                update_other_user_visibility (widget);
         }
 }
 
@@ -340,7 +374,7 @@ add_user (GdmUserChooserWidget *widget,
                 return;
         }
 
-        size = get_icon_height_for_widget (widget);
+        size = get_icon_height_for_widget (GTK_WIDGET (widget));
         pixbuf = gdm_user_render_icon (user, size);
         if (pixbuf == NULL && widget->priv->stock_person_pixbuf != NULL) {
                 pixbuf = g_object_ref (widget->priv->stock_person_pixbuf);
@@ -359,7 +393,7 @@ add_user (GdmUserChooserWidget *widget,
         gdm_chooser_widget_add_item (GDM_CHOOSER_WIDGET (widget),
                                      gdm_user_get_user_name (user),
                                      pixbuf,
-                                     gdm_user_get_real_name (user),
+                                     gdm_user_get_display_name (user),
                                      tooltip,
                                      gdm_user_get_login_frequency (user),
                                      is_logged_in,
@@ -369,6 +403,8 @@ add_user (GdmUserChooserWidget *widget,
         if (pixbuf != NULL) {
                 g_object_unref (pixbuf);
         }
+
+        update_other_user_visibility (widget);
 }
 
 static void
@@ -400,6 +436,8 @@ on_user_removed (GdmUserManager       *manager,
 
         gdm_chooser_widget_remove_item (GDM_CHOOSER_WIDGET (widget),
                                         user_name);
+
+        update_other_user_visibility (widget);
 }
 
 static void
@@ -443,8 +481,7 @@ on_users_loaded (GdmUserManager       *manager,
                  GdmUserChooserWidget *widget)
 {
         GSList *users;
-
-        widget->priv->loaded = TRUE;
+        gboolean list_visible;
 
         g_debug ("GdmUserChooserWidget: Users loaded");
 
@@ -454,7 +491,12 @@ on_users_loaded (GdmUserManager       *manager,
                 users = g_slist_delete_link (users, users);
         }
 
-        gtk_widget_grab_focus (GTK_WIDGET (widget));
+        g_object_get (G_OBJECT (widget), "list-visible", &list_visible, NULL);
+
+        if (list_visible) {
+                gtk_widget_grab_focus (GTK_WIDGET (widget));
+        }
+        widget->priv->loaded = TRUE;
 
         gdm_chooser_widget_loaded (GDM_CHOOSER_WIDGET (widget));
 }
@@ -505,6 +547,7 @@ gdm_user_chooser_widget_constructor (GType                  type,
                                                                                                               n_construct_properties,
                                                                                                               construct_properties));
 
+        widget->priv->has_user_other = FALSE;
         widget->priv->show_normal_users = !is_user_list_disabled (widget);
 
         widget->priv->load_idle_id = g_idle_add ((GSourceFunc)load_users, widget);
@@ -580,7 +623,7 @@ get_stock_person_pixbuf (GdmUserChooserWidget *widget)
         GdkPixbuf *pixbuf;
         int        size;
 
-        size = get_icon_height_for_widget (widget);
+        size = get_icon_height_for_widget (GTK_WIDGET (widget));
 
         pixbuf = gtk_icon_theme_load_icon (widget->priv->icon_theme,
                                            DEFAULT_USER_ICON,
@@ -597,7 +640,7 @@ get_logged_in_pixbuf (GdmUserChooserWidget *widget)
         GdkPixbuf *pixbuf;
         int        size;
 
-        size = get_icon_height_for_widget (widget);
+        size = get_icon_height_for_widget (GTK_WIDGET (widget));
 
         pixbuf = gtk_icon_theme_load_icon (widget->priv->icon_theme,
                                            "emblem-default",
